@@ -7,9 +7,6 @@ licence: https://opensource.org/licenses/MIT
 
 from unicodedata import bidirectional
 
-from .binary import ceildiv
-from . import matrix
-
 try:
     from bidi.algorithm import get_display
 except ImportError:
@@ -22,6 +19,14 @@ except ImportError:
     def reshape(text):
         raise ImportError('Arabic text requires module `arabic-reshaper`; not found.')
 
+try:
+    from PIL import Image
+except ImportError:
+    Image = None
+
+from .binary import ceildiv
+
+
 
 # matrix colours
 # 0, 1 are background, foreground
@@ -29,17 +34,94 @@ except ImportError:
 _BORDER = -1
 
 
+DIRECTIONS = {
+    'n': 'normal',
+    'l': 'left-to-right',
+    'r': 'right-to-left',
+    't': 'top-to-bottom',
+    'b': 'bottom-to-top'
+}
+
+###################################################################################################
+# canvas operations
+
+def create_canvas(width, height, fill=0):
+    """Create a matrix in list format."""
+    return [
+        [fill for _ in range(width)]
+        for _ in range(height)
+    ]
+
+def scale_canvas(matrix, scale_x, scale_y):
+    """Scale a matrix in list format."""
+    return [
+        [_item  for _item in _row for _ in range(scale_x)]
+        for _row in matrix for _ in range(scale_y)
+    ]
+
+def rotate_canvas(matrix, quarter_turns=1):
+    """Scale a matrix in list format."""
+    for turn in range(quarter_turns):
+        matrix = mirror_canvas(transpose_canvas(matrix))
+    return matrix
+
+def transpose_canvas(matrix):
+    """Transpose a matrix."""
+    return list(zip(*matrix))
+
+def mirror_canvas(matrix):
+    """Mirror a matrix."""
+    return [_row[::-1] for _row in matrix]
+
+
+def blit(matrix, canvas, grid_x, grid_y, operator=max):
+    """Draw a matrix onto a canvas (leaving exising ink in place, depending on operator)."""
+    if not matrix or not canvas:
+        return canvas
+    matrix_height = len(matrix)
+    canvas_height = len(canvas)
+    canvas_width = len(canvas[0])
+    for work_y in range(matrix_height):
+        y_index = grid_y - work_y - 1
+        if 0 <= y_index < canvas_height:
+            row = canvas[y_index]
+            for work_x, ink in enumerate(matrix[matrix_height - work_y - 1]):
+                if 0 <= grid_x + work_x < canvas_width:
+                    row[grid_x + work_x] = operator(ink, row[grid_x + work_x])
+    return canvas
+
+
+def to_image(matrix, border=(32, 32, 32), paper=(0, 0, 0), ink=(255, 255, 255)):
+    """Convert matrix to image."""
+    if not Image:
+        raise ImportError('Rendering to image requires PIL module.')
+    height = len(matrix)
+    if height:
+        width = len(matrix[0])
+    else:
+        width = 0
+    img = Image.new('RGB', (width, height), border)
+    img.putdata([{-1: border, 0: paper, 1: ink}[_pix] for _row in matrix for _pix in _row])
+    return img
+
+def to_text(matrix, *, border=' ', paper='-', ink='@', line_break='\n'):
+    """Convert matrix to text."""
+    colourdict = {-1: border, 0: paper, 1: ink}
+    return line_break.join(''.join(colourdict[_pix] for _pix in _row) for _row in matrix)
+
+
+
 ###################################################################################################
 # text rendering
 
 def render_text(
-        font, text, ink='@', paper='-', *,
+        font, text, ink='@', paper='.', *,
         margin=(0, 0), scale=(1, 1), rotate=0,
         direction='',
         missing='default'
     ):
     """Render text string to text bitmap."""
-    return matrix.to_text(
+    return to_text(
         render(
             font, text,
             margin=margin, scale=scale, rotate=rotate, direction=direction,
@@ -56,7 +138,7 @@ def render_image(
         missing='default',
     ):
     """Render text to image."""
-    return matrix.to_image(
+    return to_image(
         render(
             font, text,
             margin=margin, scale=scale, rotate=rotate, direction=direction,
@@ -66,26 +148,28 @@ def render_image(
     )
 
 def render(
-        font, text, *, margin=(0, 0), scale=(1, 1), rotate=0, direction='', missing='default'
+        font, text, *, margin=(0, 0), scale=(1, 1), rotate=0, direction='',
+        missing='default'
     ):
     """Render text string to bitmap."""
-    direction, align = _get_direction(text, direction)
+    direction, line_direction, align = _get_direction(text, direction)
     # get glyphs for rendering
-    glyphs = _get_text_glyphs(font, text, direction=direction, missing=missing)
+    glyphs = _get_text_glyphs(font, text, direction, line_direction, missing)
     margin_x, margin_y = margin
-    if direction == 'top-to-bottom':
+    if direction in ('top-to-bottom', 'bottom-to-top'):
         canvas = _get_canvas_vertical(font, glyphs, margin_x, margin_y)
-        canvas = _render_vertical(font, glyphs, canvas, margin_x, margin_y)
+        canvas = _render_vertical(font, glyphs, canvas, margin_x, margin_y, align)
     else:
         canvas = _get_canvas_horizontal(font, glyphs, margin_x, margin_y)
         canvas = _render_horizontal(font, glyphs, canvas, margin_x, margin_y, align)
-    scaled = matrix.scale(canvas, *scale)
-    rotated = matrix.rotate(scaled, rotate)
+    scaled = scale_canvas(canvas, *scale)
+    rotated = rotate_canvas(scaled, rotate)
     return rotated
 
 def _render_horizontal(font, glyphs, canvas, margin_x, margin_y, align):
     # descent-line of the bottom-most row is at bottom margin
-    # if a glyph extends below the descent line or left of the orgin, it may draw into the margin
+    # if a glyph extends below the descent line or left of the origin,
+    # it may draw into the margin
     # raster_size.y moves from canvas origin to raster origin (bottom line)
     baseline = font.ascent
     for glyph_row in glyphs:
@@ -110,25 +194,38 @@ def _render_horizontal(font, glyphs, canvas, margin_x, margin_y, align):
         else:
             start = margin_x
         for glyph, x, y in zip(glyph_row, grid_x, grid_y):
-            # add ink, taking into account there may be ink already in case of negative bearings
-            matrix.blit(glyph.as_matrix(), canvas, start + x, margin_y + y)
+            # add ink, taking into account there may be ink already
+            # in case of negative bearings
+            blit(glyph.as_matrix(), canvas, start + x, margin_y + y)
         # move to next line
         baseline += font.line_height
     return canvas
 
-def _render_vertical(font, glyphs, canvas, margin_x, margin_y):
+def _render_vertical(font, glyphs, canvas, margin_x, margin_y, align):
     # central axis (with leftward bias)
-    baseline = margin_x + int(font.line_width / 2)
+    baseline = int(font.line_width / 2)
     # default is ttb right-to-left
-    for glyph_row in reversed(glyphs):
+    for glyph_row in glyphs:
         y = 0
+        grid_x, grid_y = [], []
         for glyph in glyph_row:
             # advance origin to next glyph
             y += font.top_bearing + glyph.advance_height + font.bottom_bearing
-            grid_y = margin_y + (font.top_bearing + glyph.top_bearing + y)
-            grid_x = baseline - int(glyph.width / 2) - (font.shift_left + glyph.shift_left)
-            # add ink, taking into account there may be ink already in case of negative bearing
-            matrix.blit(glyph.as_matrix(), canvas, grid_x, grid_y)
+            grid_y.append(
+                y - (font.bottom_bearing + glyph.bottom_bearing)
+            )
+            grid_x.append(
+                baseline - int(glyph.width / 2) - (font.shift_left + glyph.shift_left)
+            )
+        if align == 'bottom':
+            start = len(canvas) - margin_y - y
+        else:
+            start = margin_y
+
+        for glyph, x, y in zip(glyph_row, grid_x, grid_y):
+            # add ink, taking into account there may be ink already
+            # in case of negative bearings
+            blit(glyph.as_matrix(), canvas, margin_x + x, start + y)
         # move to next line
         baseline += font.line_width
     return canvas
@@ -139,15 +236,19 @@ def _get_canvas_horizontal(font, glyphs, margin_x, margin_y):
     width = 2 * margin_x
     if glyphs:
         width += max(
-            sum(font.left_bearing + _glyph.advance_width + font.right_bearing for _glyph in _row)
+            sum(
+                font.left_bearing + _glyph.advance_width + font.right_bearing
+                for _glyph in _row
+            )
             for _row in glyphs
         )
     # find required height - margins plus line height for each row
     # descent-line of the bottom-most row is at bottom margin
     # ascent-line of top-most row is at top margin
-    # if a glyph extends below the descent line or left of the origin, it may draw into the margin
+    # if a glyph extends below the descent line or left of the origin,
+    # it may draw into the margin
     height = 2 * margin_y + font.pixel_size + font.line_height * (len(glyphs)-1)
-    return matrix.create(width, height)
+    return create_canvas(width, height)
 
 def _get_canvas_vertical(font, glyphs, margin_x, margin_y):
     """Create canvas of the right size."""
@@ -155,30 +256,66 @@ def _get_canvas_vertical(font, glyphs, margin_x, margin_y):
     height = 2 * margin_y
     if glyphs:
         height += max(
-            sum(font.top_bearing + _glyph.advance_height + font.bottom_bearing for _glyph in _row)
+            sum(
+                font.top_bearing + _glyph.advance_height + font.bottom_bearing
+                for _glyph in _row
+            )
             for _row in glyphs
         )
     width = 2 * margin_x + font.line_width * len(glyphs)
-    return matrix.create(width, height)
+    return create_canvas(width, height)
 
 
 def _get_direction(text, direction):
     """Get direction and alignment."""
-    if direction == 'top-to-bottom':
-        return direction, 'right'
     isstr = isinstance(text, str)
     if not direction:
         if isstr:
-            direction = 'normal'
+            direction = 'n'
         else:
-            direction = 'left-to-right'
-    if direction not in ('normal', 'reverse', 'right-to-left', 'left-to-right'):
-        raise ValueError(f'Unsupported writing direction `{direction}`')
-    left_align = True
-    if direction in ('normal', 'reverse'):
+            direction = 'l'
+    direction = direction.lower()
+    # get line advance direction if given
+    direction, _, line_direction = direction.partition(' ')
+    try:
+        direction = DIRECTIONS[direction[0]]
+    except KeyError:
+        raise ValueError(
+            'Writing direction must be one of '
+            + ', '.join(
+                f'`{_k}`==`{_v}`'
+                for _k, _v in DIRECTIONS.items()
+            )
+            + f'; not `{direction}`.'
+        )
+    if not line_direction or line_direction.startswith('n'):
+        if direction == 'top-to-bottom':
+            line_direction = 'r'
+        else:
+            line_direction = 't'
+    try:
+        line_direction = DIRECTIONS[line_direction[0]]
+    except KeyError:
+        raise ValueError(
+            'Line direction must be one of '
+            + ', '.join(
+                f'`{_k}`==`{_v}`'
+                for _k, _v in DIRECTIONS.items()
+            )
+            + f'; not `{direction}`.'
+        )
+    if direction == 'left-to-right':
+        align = 'left'
+    elif direction == 'right-to-left':
+        align = 'right'
+    elif direction == 'normal':
         if not isstr:
-            raise ValueError(f'Writing direction `{direction}` only supported for Unicode text.')
-        # determine alignment by the class of the first directional character encountered
+            raise ValueError(
+                f'Writing direction `{direction}` only supported for Unicode text.'
+            )
+        # determine alignment
+        # by the class of the first directional character encountered
+        align = 'left'
         for c in text:
             try:
                 bidicls = bidirectional(c)[0]
@@ -187,47 +324,52 @@ def _get_direction(text, direction):
             if bidicls == 'L':
                 break
             if bidicls in ('R', 'A'):
-                left_align = False
+                align = 'right'
                 break
-        if direction == 'reverse':
-            left_align = not left_align
-    elif direction == 'right-to-left':
-        left_align = False
-    return direction, 'left' if left_align else 'right'
-
-
-def _get_text_glyphs(font, text, direction, missing='raise'):
-    """Get tuple of tuples of glyphs (by line) from str or bytes/codepoints input."""
-    if direction != 'top-to-bottom':
-        if isinstance(text, str):
-            # reshape Arabic glyphs to contextual forms
-            try:
-                text = reshape(text)
-            except ImportError as e:
-                # check common Arabic range - is there anything to reshape?
-                if any(ord(_c) in range(0x600, 0x700) for _c in text):
-                    logging.warning(e)
-            # put characters in visual order instead of logical
-            if direction in ('normal', 'reverse'):
-                # decide direction based on bidi algorithm
-                text = get_display(text)
-        if direction in ('right-to-left', 'reverse'):
-            # reverse writing order
-            text = ''.join(reversed(text))
-    if isinstance(text, str):
-        max_length = max(len(_c) for _c in font.get_chars())
-        type_conv = str
+    elif direction == 'bottom-to-top':
+        align = 'bottom'
     else:
-        max_length = max(len(_cp) for _cp in font.get_codepoints())
-        type_conv = tuple
+        align = 'top'
+    return direction, line_direction, align
+
+
+def _get_text_glyphs(font, text, direction, line_direction, missing='raise'):
+    """
+    Get tuple of tuples of glyphs (by line) from str or bytes/codepoints input.
+    Glyphs are reordered so that they can be rendered ltr ttb or ttb ltr
+    """
+    if isinstance(text, str) and direction not in ('top-to-bottom', 'bottom-to-top'):
+        # reshape Arabic glyphs to contextual forms
+        try:
+            text = reshape(text)
+        except ImportError as e:
+            # check common Arabic range - is there anything to reshape?
+            if any(ord(_c) in range(0x600, 0x700) for _c in text):
+                logging.warning(e)
+        # put characters in visual order instead of logical
+        if direction == 'normal':
+            # decide direction based on bidi algorithm
+            text = get_display(text)
+    lines = text.splitlines()
+    if direction in ('right-to-left', 'bottom-to-top'):
+        # reverse glyph order for rendering
+        lines = tuple(_row[::-1] for _row in lines)
+    if line_direction in ('right-to-left', 'bottom-to-top'):
+        # reverse line order for rendering
+        lines = lines[::-1]
     return tuple(
-        tuple(_iter_labels(font, type_conv(_line), max_length, missing))
-        for _line in text.splitlines()
+        tuple(_iter_labels(font, _line, missing))
+        for _line in lines
     )
 
-def _iter_labels(font, labels, max_length, missing='raise'):
-    """Iterate over labels, yielding glyphs."""
-    remaining = labels
+def _iter_labels(font, text, missing='raise'):
+    """Iterate over labels in text, yielding glyphs. text may be str or bytes."""
+    if isinstance(text, str):
+        labelset = font.get_chars()
+    else:
+        labelset = font.get_codepoints()
+    max_length = max(len(_c) for _c in labelset)
+    remaining = text
     while remaining:
         # try multibyte clusters first
         for try_len in range(max_length, 1, -1):
@@ -255,7 +397,7 @@ def chart_image(
     ):
     """Create font chart as image."""
     canvas = chart(font, columns, margin, padding, scale, order, direction)
-    return matrix.to_image(canvas, border=border, paper=paper, ink=ink)
+    return to_image(canvas, border=border, paper=paper, ink=ink)
 
 def chart_text(
         font,
@@ -265,7 +407,7 @@ def chart_text(
     ):
     """Create font chart as text."""
     canvas = chart(font, columns, margin, padding, scale, order, direction)
-    return matrix.to_text(canvas, border=border, paper=paper, ink=ink)
+    return to_text(canvas, border=border, paper=paper, ink=ink)
 
 def traverse_chart(columns, rows, order, direction):
     """Traverse a glyph chart in the specified order and directions."""
@@ -311,7 +453,7 @@ def chart(
     # determine image geometry
     width = columns * step_x + 2 * margin_x - padding_x
     height = rows * step_y + 2 * margin_y - padding_y
-    canvas = matrix.create(width, height, _BORDER)
+    canvas = create_canvas(width, height, _BORDER)
     # output glyphs
     traverse = traverse_chart(columns, rows, order, direction)
     for glyph, pos in zip(font.glyphs, traverse):
@@ -319,8 +461,8 @@ def chart(
             continue
         row, col = pos
         mx = glyph.as_matrix()
-        mx = matrix.scale(mx, scale_x, scale_y)
+        mx = scale_canvas(mx, scale_x, scale_y)
         left = margin_x + col*step_x + glyph.left_bearing
         bottom = margin_y + (row+1)*step_y - padding_y - glyph.shift_up
-        matrix.blit(mx, canvas, left, bottom)
+        blit(mx, canvas, left, bottom)
     return canvas
