@@ -134,6 +134,10 @@ class DefaultProps(Props):
     where field_1 has an implied default, int() == 0
     """
 
+    # set to True when defaults have been set for a given type
+    # on first instatntiation
+    _init = False
+
     def __init__(self, *args, **kwargs):
         # disable cacheing while building the object
         self._frozen = False
@@ -142,13 +146,16 @@ class DefaultProps(Props):
         # note that we're changing the *class* namespace on the *instance* initialiser
         # which feels a bit hacky
         # but this will be a no-op after the first instance has initialised
-        self._set_defaults()
+        if not type(self)._init:
+            self._set_defaults()
+            type(self)._init = True
         # use Props.__setitem__ for value conversion
         # we use None to *unset* properties
-        for field, value in kwargs.items():
-            if value is not None:
-                field = normalise_property(field)
-                setattr(self, field, value)
+        [
+            setattr(self, normalise_property(field), value)
+            for field, value in kwargs.items()
+            if value is not None
+        ]
         # enable cacheing
         self._cache = {}
         self._frozen = True
@@ -212,7 +219,6 @@ def writable_property(arg=None, *, field=None):
     fn = arg
     field = field or fn.__name__
     field = normalise_property(field)
-
     cached_fn = delayed_cache(fn)
 
     @wraps(fn)
@@ -229,15 +235,7 @@ def writable_property(arg=None, *, field=None):
     def _setter(self, value):
         if self._frozen:
             raise ValueError('Cannot set property on frozen object.')
-        try:
-            calc_value = fn(self)
-        except AttributeError:
-            pass
-        else:
-            if value == calc_value:
-                logging.debug(f'Overridable property {field}={value} consistently set.')
-            else:
-                logging.info(f'Setting overridable property {field}={value} (was {calc_value}).')
+        logging.info(f'Setting overridable property {field}={value}.')
         vars(self)[field] = value
 
     return property(_getter, _setter)
@@ -253,16 +251,7 @@ def checked_property(fn):
     def _setter(self, value):
         if self._frozen:
             raise ValueError('Cannot set property on frozen object.')
-        try:
-            calc_value = fn(self)
-        except Exception as e:
-            logging.warning(f'Could not check value of {field}: {e}')
-        if value == calc_value:
-            logging.debug(f'Non-overridable property {field}={value} consistently set.')
-        else:
-            logging.warning(
-                f'Non-overridable property {field}={calc_value} cannot be set to {value}.'
-            )
+        logging.info(f'Non-overridable property {field} cannot be set to {value}; ignored.')
 
     return property(_getter, _setter)
 
